@@ -1,54 +1,74 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+
 import datetime
 import warnings
+warnings.filterwarnings('ignore', "", ImportWarning)
+from locale import CHAR_MAX
+warnings.resetwarnings()
 from ast import literal_eval
 
-from ..model import ModelView, ModelSQL, fields
+from sql import Table
+
+from ..model import ModelView, ModelSQL, DeactivableMixin, fields, Check
 from ..cache import Cache
 from ..tools import datetime_strftime
 from ..transaction import Transaction
 from ..pool import Pool
 from .time_locale import TIME_LOCALE
 
-warnings.filterwarnings('ignore', "", ImportWarning)
-from locale import CHAR_MAX
-warnings.resetwarnings()
+Transaction.cache_keys.add('translate_name')
 
 __all__ = [
     'Lang',
     ]
 
 
-class Lang(ModelSQL, ModelView):
+class Lang(DeactivableMixin, ModelSQL, ModelView):
     "Language"
     __name__ = "ir.lang"
     name = fields.Char('Name', required=True, translate=True)
     code = fields.Char('Code', required=True,
         help="RFC 4646 tag: http://tools.ietf.org/html/rfc4646")
     translatable = fields.Boolean('Translatable')
-    active = fields.Boolean('Active')
+    parent = fields.Char("Parent Code", help="Code of the exceptional parent")
     direction = fields.Selection([
             ('ltr', 'Left-to-right'),
             ('rtl', 'Right-to-left'),
             ], 'Direction', required=True)
 
-    #date
+    # date
     date = fields.Char('Date', required=True)
 
-    #number
+    # number
     grouping = fields.Char('Grouping', required=True)
     decimal_point = fields.Char('Decimal Separator', required=True)
     thousands_sep = fields.Char('Thousands Separator')
 
+    # monetary formatting
+    mon_grouping = fields.Char('Grouping', required=True)
+    mon_decimal_point = fields.Char('Decimal Separator', required=True)
+    mon_thousands_sep = fields.Char('Thousands Separator')
+    p_sign_posn = fields.Integer('Positive Sign Position', required=True)
+    n_sign_posn = fields.Integer('Negative Sign Position', required=True)
+    positive_sign = fields.Char('Positive Sign')
+    negative_sign = fields.Char('Negative Sign')
+    p_cs_precedes = fields.Boolean('Positive Currency Symbol Precedes')
+    n_cs_precedes = fields.Boolean('Negative Currency Symbol Precedes')
+    p_sep_by_space = fields.Boolean('Positive Separate by Space')
+    n_sep_by_space = fields.Boolean('Negative Separate by Space')
+
     _lang_cache = Cache('ir.lang')
+    _code_cache = Cache('ir.lang.code', context=False)
 
     @classmethod
     def __setup__(cls):
         super(Lang, cls).__setup__()
+
+        table = cls.__table__()
         cls._sql_constraints += [
             ('check_decimal_point_thousands_sep',
-                'CHECK(decimal_point != thousands_sep)',
+                Check(table, table.decimal_point != table.thousands_sep),
                 'decimal_point and thousands_sep must be different!'),
             ]
         cls._error_messages.update({
@@ -91,10 +111,6 @@ class Lang(ModelSQL, ModelView):
                     or record2['name'])
         return res
 
-    @staticmethod
-    def default_active():
-        return True
-
     @staticmethod
     def default_translatable():
         return False
@@ -119,6 +135,50 @@ class Lang(ModelSQL, ModelView):
     def default_thousands_sep():
         return ','
 
+    @classmethod
+    def default_mon_grouping(cls):
+        return '[]'
+
+    @classmethod
+    def default_mon_thousands_sep(cls):
+        return ','
+
+    @classmethod
+    def default_mon_decimal_point(cls):
+        return '.'
+
+    @classmethod
+    def default_p_sign_posn(cls):
+        return 1
+
+    @classmethod
+    def default_n_sign_posn(cls):
+        return 1
+
+    @classmethod
+    def default_negative_sign(cls):
+        return '-'
+
+    @classmethod
+    def default_positive_sign(cls):
+        return ''
+
+    @classmethod
+    def default_p_cs_precedes(cls):
+        return True
+
+    @classmethod
+    def default_n_cs_precedes(cls):
+        return True
+
+    @classmethod
+    def default_p_sep_by_space(cls):
+        return False
+
+    @classmethod
+    def default_n_sep_by_space(cls):
+        return False
+
     @classmethod
     def validate(cls, languages):
         super(Lang, cls).validate(languages)
@@ -132,16 +192,17 @@ class Lang(ModelSQL, ModelView):
         Check if grouping is list of numbers
         '''
         for lang in langs:
-            try:
-                grouping = literal_eval(lang.grouping)
-                for i in grouping:
-                    if not isinstance(i, int):
-                        raise
-            except Exception:
-                cls.raise_user_error('invalid_grouping', {
-                        'grouping': lang.grouping,
-                        'language': lang.rec_name,
-                        })
+            for grouping in [lang.grouping, lang.mon_grouping]:
+                try:
+                    grouping = literal_eval(grouping)
+                    for i in grouping:
+                        if not isinstance(i, int):
+                            raise
+                except Exception:
+                    cls.raise_user_error('invalid_grouping', {
+                            'grouping': grouping,
+                            'language': lang.rec_name,
+                            })
 
     @classmethod
     def check_date(cls, langs):
@@ -149,9 +210,9 @@ class Lang(ModelSQL, ModelView):
         Check the date format
         '''
         for lang in langs:
+            date = lang.date
             try:
-                datetime_strftime(datetime.datetime.now(),
-                        lang.date.encode('utf-8'))
+                datetime_strftime(datetime.datetime.now(), date)
             except Exception:
                 cls.raise_user_error('invalid_date', {
                         'format': lang.date,
@@ -211,6 +272,7 @@ class Lang(ModelSQL, ModelView):
         cls._lang_cache.clear()
         languages = super(Lang, cls).create(vlist)
         Translation._get_language_cache.clear()
+        _parents.clear()
         return languages
 
     @classmethod
@@ -219,8 +281,10 @@ class Lang(ModelSQL, ModelView):
         Translation = pool.get('ir.translation')
         # Clear cache
         cls._lang_cache.clear()
+        cls._code_cache.clear()
         super(Lang, cls).write(langs, values, *args)
         Translation._get_language_cache.clear()
+        _parents.clear()
 
     @classmethod
     def delete(cls, langs):
@@ -232,11 +296,28 @@ class Lang(ModelSQL, ModelView):
                 cls.raise_user_error('delete_default')
         # Clear cache
         cls._lang_cache.clear()
+        cls._code_cache.clear()
         super(Lang, cls).delete(langs)
         Translation._get_language_cache.clear()
+        _parents.clear()
 
-    @staticmethod
-    def _group(lang, s, monetary=None):
+    @classmethod
+    def get(cls, code=None):
+        "Return language instance for the code or the transaction language"
+        if code is None:
+            code = Transaction().language
+        lang_id = cls._code_cache.get(code)
+        if not lang_id:
+            with Transaction().set_context(active_test=False):
+                lang, = cls.search([
+                        ('code', '=', code),
+                        ])
+            cls._code_cache.set(code, lang.id)
+        else:
+            lang = cls(lang_id)
+        return lang
+
+    def _group(self, s, monetary=False):
         # Code from _group in locale.py
 
         # Iterate over grouping intervals
@@ -254,11 +335,11 @@ class Lang(ModelSQL, ModelView):
                 last_interval = interval
 
         if monetary:
-            thousands_sep = monetary.mon_thousands_sep
-            grouping = literal_eval(monetary.mon_grouping)
+            thousands_sep = self.mon_thousands_sep
+            grouping = literal_eval(self.mon_grouping)
         else:
-            thousands_sep = lang.thousands_sep
-            grouping = literal_eval(lang.grouping)
+            thousands_sep = self.thousands_sep
+            grouping = literal_eval(self.grouping)
         if not grouping:
             return (s, 0)
         if s[-1] == ' ':
@@ -285,8 +366,7 @@ class Lang(ModelSQL, ModelView):
             len(thousands_sep) * (len(groups) - 1)
         )
 
-    @classmethod
-    def format(cls, lang, percent, value, grouping=False, monetary=None,
+    def format(self, percent, value, grouping=False, monetary=False,
             *additional):
         '''
         Returns the lang-aware substitution of a %? specifier (percent).
@@ -305,13 +385,6 @@ class Lang(ModelSQL, ModelView):
                 amount -= 1
             return s[lpos:rpos + 1]
 
-        if not lang:
-            lang = cls(
-                decimal_point=cls.default_decimal_point(),
-                thousands_sep=cls.default_thousands_sep(),
-                grouping=cls.default_grouping(),
-                )
-
         # this is only for one-percent-specifier strings
         # and this should be checked
         if percent[0] != '%':
@@ -326,34 +399,26 @@ class Lang(ModelSQL, ModelView):
             seps = 0
             parts = formatted.split('.')
             if grouping:
-                parts[0], seps = cls._group(lang, parts[0], monetary=monetary)
+                parts[0], seps = self._group(parts[0], monetary=monetary)
             if monetary:
-                decimal_point = monetary.mon_decimal_point
+                decimal_point = self.mon_decimal_point
             else:
-                decimal_point = lang.decimal_point
+                decimal_point = self.decimal_point
             formatted = decimal_point.join(parts)
             if seps:
                 formatted = _strip_padding(formatted, seps)
         elif percent[-1] in 'diu':
             if grouping:
-                formatted, seps = cls._group(lang, formatted,
-                    monetary=monetary)
+                formatted, seps = self._group(formatted, monetary=monetary)
             if seps:
                 formatted = _strip_padding(formatted, seps)
         return formatted
 
-    @classmethod
-    def currency(cls, lang, val, currency, symbol=True, grouping=False):
+    def currency(self, val, currency, symbol=True, grouping=False):
         """
         Formats val according to the currency settings in lang.
         """
         # Code from currency in locale.py
-        if not lang:
-            lang = cls(
-                decimal_point=cls.default_decimal_point(),
-                thousands_sep=cls.default_thousands_sep(),
-                grouping=cls.default_grouping(),
-                )
 
         # check for illegal values
         digits = currency.digits
@@ -361,26 +426,26 @@ class Lang(ModelSQL, ModelView):
             raise ValueError("Currency formatting is not possible using "
                              "the 'C' locale.")
 
-        s = cls.format(lang, '%%.%if' % digits, abs(val), grouping,
-                monetary=currency)
+        s = self.format(
+            '%%.%if' % digits, abs(val), grouping, monetary=True)
         # '<' and '>' are markers if the sign must be inserted
         # between symbol and value
         s = '<' + s + '>'
 
         if symbol:
             smb = currency.symbol
-            precedes = (val < 0 and currency.n_cs_precedes
-                or currency.p_cs_precedes)
-            separated = (val < 0 and currency.n_sep_by_space
-                or currency.p_sep_by_space)
+            precedes = (val < 0 and self.n_cs_precedes
+                or self.p_cs_precedes)
+            separated = (val < 0 and self.n_sep_by_space
+                or self.p_sep_by_space)
 
             if precedes:
                 s = smb + (separated and ' ' or '') + s
             else:
                 s = s + (separated and ' ' or '') + smb
 
-        sign_pos = val < 0 and currency.n_sign_posn or currency.p_sign_posn
-        sign = val < 0 and currency.negative_sign or currency.positive_sign
+        sign_pos = val < 0 and self.n_sign_posn or self.p_sign_posn
+        sign = val < 0 and self.negative_sign or self.positive_sign
 
         if sign_pos == 0:
             s = '(' + s + ')'
@@ -399,18 +464,33 @@ class Lang(ModelSQL, ModelView):
 
         return s.replace('<', '').replace('>', '')
 
-    @staticmethod
-    def strftime(datetime, code, format):
+    def strftime(self, datetime, format=None):
         '''
         Convert datetime to a string as specified by the format argument.
         '''
+        if format is None:
+            format = self.date
+        code = self.code[:2]
         if code in TIME_LOCALE:
             for f, i in (('%a', 6), ('%A', 6), ('%b', 1), ('%B', 1)):
                 format = format.replace(f,
                         TIME_LOCALE[code][f][datetime.timetuple()[i]])
             format = format.replace('%p',
                 TIME_LOCALE[code]['%p'][datetime.timetuple()[3] < 12 and 0
-                    or 1]).encode('utf-8')
-        else:
-            format = format.encode('utf-8')
-        return datetime_strftime(datetime, format).decode('utf-8')
+                    or 1])
+        return datetime_strftime(datetime, format)
+
+
+def get_parent_language(code):
+    if code not in _parents:
+        # Use SQL because it is used by load_module_graph
+        cursor = Transaction().connection.cursor()
+        lang = Table('ir_lang')
+        cursor.execute(*lang.select(lang.code, lang.parent))
+        _parents.update(cursor.fetchall())
+    if _parents.get(code):
+        return _parents[code]
+    for sep in ['@', '_']:
+        if sep in code:
+            return code.rsplit(sep, 1)[0]
+_parents = {}
