@@ -981,7 +981,8 @@ class ModelStorage(Model):
                     in_max = Transaction().database.IN_MAX
                     count = in_max // 10
                     new_domains = {}
-                    for sub_domains in grouped_slice(list(domains.keys()), count):
+                    for sub_domains in grouped_slice(
+                            list(domains.keys()), count):
                         grouped_domain = ['OR']
                         grouped_records = []
                         for d in sub_domains:
@@ -992,7 +993,11 @@ class ModelStorage(Model):
                                 break
                             grouped_domain.append(
                                 [('id', 'in', [r.id for r in relations]), d])
-                        new_domains[freeze(grouped_domain)] = grouped_records
+                        else:
+                            new_domains[freeze(grouped_domain)] = \
+                                grouped_records
+                            continue
+                        break
                     else:
                         domains = new_domains
             else:
@@ -1341,7 +1346,7 @@ class ModelStorage(Model):
             if field.context:
                 eval_fields = fields.get_eval_fields(field.context)
                 for context_field_name in eval_fields:
-                    if context_field_name in field.depends:
+                    if context_field_name not in field.depends:
                         continue
                     context_field = self._fields.get(context_field_name)
                     if context_field not in ffields:
