@@ -229,8 +229,6 @@ class Report(URLMixin, PoolBase):
             def __str__(self):
                 return '%s,%s' % (Model.__name__, self.id)
 
-            def __unicode__(self):
-                return '%s,%s' % (Model.__name__, self.id)
         return [TranslateModel(id) for id in ids]
 
     @classmethod
@@ -308,7 +306,8 @@ class Report(URLMixin, PoolBase):
         path = os.path.join(
             dtemp, report.report_name + os.extsep + input_format)
         oext = FORMAT2EXT.get(output_format, output_format)
-        with open(path, 'wb+') as fp:
+        mode = 'w+' if isinstance(data, str) else 'wb+'
+        with open(path, mode) as fp:
             fp.write(data)
         try:
             cmd = ['soffice',
